@@ -5,6 +5,7 @@ import warnings
 import inspect
 import re
 import collections
+import weakref
 
 def Observe(*args):
     return SelfExporter.default_model.add_observable(*args)
@@ -63,7 +64,8 @@ class SelfExporter(object):
             if obj.name is None:
                 if SelfExporter.target_module == sys.modules['__main__']:
                     # user ran model .py directly
-                    model_filename = inspect.getfile(sys.modules['__main__'])
+                    model_path = inspect.getfile(sys.modules['__main__'])
+                    model_filename = os.path.basename(model_path)
                     module_name = re.sub(r'\.py$', '', model_filename)
                 elif SelfExporter.target_module is not None:
                     # model is imported by some other script (typical case)
@@ -85,12 +87,14 @@ class SelfExporter(object):
 
 
 class Component(object):
+
     """The base class for all the things contained within a model."""
 
     def __init__(self, name, _export=True):
         if not re.match(r'[_a-z][_a-z0-9]*\Z', name, re.IGNORECASE):
             raise InvalidComponentNameError(name)
         self.name = name
+        self.model = None  # to be set in Model.add_component
         if _export:
             try:
                 SelfExporter.export(self)
@@ -102,6 +106,8 @@ class Component(object):
 
 class Model(object):
 
+    """Container for monomers, compartments, parameters, and rules."""
+
     def __init__(self, name=None, _export=True):
         self.name = name
         self.monomers = ComponentSet()
@@ -177,11 +183,13 @@ class Model(object):
         # We have 4 containers for the 4 types of components. This code determines the right one
         # based on the class of the object being added.  It tries to be defensive against reasonable
         # errors, but still seems sort of fragile.
+        # FIXME: this breaks for subclasses of the Component classes
         container_name = type(other).__name__.lower() + 's'
         container = getattr(self, container_name, None)
         if not isinstance(other, Component) or not isinstance(container, ComponentSet):
             raise Exception("Tried to add component of unknown type '%s' to model" % type(other))
         container.add(other)
+        other.model = weakref.proxy(self)
 
     def add_observable(self, name, reaction_pattern):
         try:
@@ -219,8 +227,8 @@ class Model(object):
 
 
 class Monomer(Component):
-    """The Monomer class creates monomers with the specified sites, state-sites, and compartment
-    """
+
+    """Model component representing a protein or other molecule."""
 
     def __init__(self, name, sites=[], site_states={}, _export=True):
         Component.__init__(self, name, _export)
@@ -263,6 +271,12 @@ class Monomer(Component):
 
 class MonomerAny(Monomer):
 
+    """
+    A wildcard monomer which matches any species.
+
+    This is only needed where you would use a '+' in BNG.
+    """
+
     def __init__(self):
         # don't call Monomer.__init__ since this doesn't want
         # Component stuff and has no user-accessible API
@@ -279,6 +293,12 @@ class MonomerAny(Monomer):
 
 class MonomerWild(Monomer):
 
+    """
+    A wildcard monomer which matches any species, or nothing (no bond).
+
+    This is only needed where you would use a '?' in BNG.
+    """
+
     def __init__(self):
         # don't call Monomer.__init__ since this doesn't want
         # Component stuff and has no user-accessible API
@@ -295,6 +315,9 @@ class MonomerWild(Monomer):
 
 class MonomerPattern(object):
 
+    """A pattern which matches instances of a given monomer, possibly with
+    restrictions on the state of certain sites."""
+
     def __init__(self, monomer, site_conditions, compartment):
         # ensure all keys in site_conditions are sites in monomer
         unknown_sites = [site for site in site_conditions.keys() if site not in monomer.sites_dict]
@@ -302,7 +325,6 @@ class MonomerPattern(object):
             raise Exception("MonomerPattern with unknown sites in " + str(monomer) + ": " + str(unknown_sites))
 
         # ensure each value is one of: None, integer, list of integers, string, (string,integer), (string,WILD), ANY
-        # FIXME: support state sites
         invalid_sites = []
         for (site, state) in site_conditions.items():
             # pass through to next iteration if state type is ok
@@ -336,14 +358,11 @@ class MonomerPattern(object):
         'Concrete' means the pattern satisfies ALL of the following:
         1. All sites have specified conditions
         2. If the model uses compartments, the compartment is specified.
-
         """
         # 1.
         sites_ok = self.is_site_concrete()
         # 2.
-        # FIXME accessing the model via SelfExporter.default_model is a temporary hack - all model
-        #   components (Component subclasses?) need weak refs to their parent model.
-        compartment_ok = self.compartment is not None or not SelfExporter.default_model.compartments
+        compartment_ok = not self.monomer.model.compartments or self.compartment
         return compartment_ok and sites_ok
 
     def is_site_concrete(self):
@@ -379,13 +398,13 @@ class MonomerPattern(object):
 
     def __rshift__(self, other):
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
-            return (self, other, False)
+            return RuleExpression(self, other, False)
         else:
             return NotImplemented
 
     def __ne__(self, other):
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
-            return (self, other, True)
+            return RuleExpression(self, other, True)
         else:
             return NotImplemented
 
@@ -412,8 +431,12 @@ class MonomerPattern(object):
 
 
 class ComplexPattern(object):
-    """Represents a bound set of MonomerPatterns, i.e. a complex.  In
-    BNG terms, a list of patterns combined with the '.' operator)."""
+
+    """
+    A bound set of MonomerPatterns, i.e. a pattern to match a complex.
+
+    In BNG terms, a list of patterns combined with the '.' operator.
+    """
 
     def __init__(self, monomer_patterns, compartment, match_once=False):
         # ensure compartment is a Compartment
@@ -430,7 +453,6 @@ class ComplexPattern(object):
         'Concrete' means the pattern satisfies ANY of the following:
         1. All monomer patterns are concrete
         2. The compartment is specified AND all monomer patterns are site-concrete
-
         """
         # 1.
         mp_concrete_ok = all(mp.is_concrete() for mp in self.monomer_patterns)
@@ -452,10 +474,12 @@ class ComplexPattern(object):
             sorted((mp.monomer, mp.site_conditions) for mp in other.monomer_patterns)
 
     def copy(self):
-        """Implement our own brand of shallow copy.
+        """
+        Implement our own brand of shallow copy.
 
         The new object will have references to the original compartment, and
-        copies of the monomer_patterns."""
+        copies of the monomer_patterns.
+        """
         return ComplexPattern([mp() for mp in self.monomer_patterns], self.compartment, self.match_once)
 
     def __add__(self, other):
@@ -480,13 +504,13 @@ class ComplexPattern(object):
 
     def __rshift__(self, other):
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
-            return (self, other, False)
+            return RuleExpression(self, other, False)
         else:
             return NotImplemented
 
     def __ne__(self, other):
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
-            return (self, other, True)
+            return RuleExpression(self, other, True)
         else:
             return NotImplemented
 
@@ -509,9 +533,14 @@ class ComplexPattern(object):
 
 
 class ReactionPattern(object):
-    """Represents a complete pattern for the product or reactant side
-    of a rule.  Essentially a thin wrapper around a list of
-    ComplexPatterns."""
+
+    """
+    A pattern for the entire product or reactant side of a rule.
+
+    Essentially a thin wrapper around a list of ComplexPatterns. In BNG terms, a
+    list of complex patterns combined with the '+' operator.
+
+    """
 
     def __init__(self, complex_patterns):
         self.complex_patterns = complex_patterns
@@ -527,14 +556,14 @@ class ReactionPattern(object):
     def __rshift__(self, other):
         """Irreversible reaction"""
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
-            return (self, other, False)
+            return RuleExpression(self, other, False)
         else:
             return NotImplemented
 
     def __ne__(self, other):
         """Reversible reaction"""
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
-            return (self, other, True)
+            return RuleExpression(self, other, True)
         else:
             return NotImplemented
 
@@ -543,9 +572,34 @@ class ReactionPattern(object):
 
 
 
+class RuleExpression(object):
+
+    """
+    A container for the reactant and product patterns of a rule expression.
+
+    Contains one ReactionPattern for each of reactants and products, and a
+    boolean indicating reversibility. This is a temporary object used to
+    implement syntactic sugar through operator overloading. The Rule constructor
+    takes an instance of this class as its first argument, but simply extracts
+    its fields and discards the object itself.
+
+    """
+
+    def __init__(self, reactant_pattern, product_pattern, is_reversible):
+        try:
+            self.reactant_pattern = as_reaction_pattern(reactant_pattern)
+        except InvalidReactionPatternException as e:
+            raise type(e)("Reactant does not look like a reaction pattern")
+        try:
+            self.product_pattern = as_reaction_pattern(product_pattern)
+        except InvalidReactionPatternException as e:
+            raise type(e)("Product does not look like a reaction pattern")
+        self.is_reversible = is_reversible
+
+
+
 def as_complex_pattern(v):
     """Internal helper to 'upgrade' a MonomerPattern to a ComplexPattern."""
-
     if isinstance(v, ComplexPattern):
         return v
     elif isinstance(v, MonomerPattern):
@@ -557,7 +611,6 @@ def as_complex_pattern(v):
 def as_reaction_pattern(v):
     """Internal helper to 'upgrade' a Complex- or MonomerPattern to a
     complete ReactionPattern."""
-
     if isinstance(v, ReactionPattern):
         return v
     else:
@@ -570,6 +623,13 @@ def as_reaction_pattern(v):
 
 class Parameter(Component):
 
+    """
+    Model component representing a named constant floating point number.
+
+    Parameters are used as reaction rate constants, compartment volumes and
+    initial (boundary) conditions for species.
+    """
+
     def __init__(self, name, value=0.0, _export=True):
         Component.__init__(self, name, _export)
         self.value = value
@@ -580,14 +640,21 @@ class Parameter(Component):
 
 
 class Compartment(Component):
-    """The Compartment class expects a "name", "parent", "dimension", and "size" variable from the
-    compartment call. name is a string, "parent" should be the name of a defined parent, or None. 
-    Dimension should be only 2 (e.g. membranes) or 3 (e.g. cytosol). The size units will depend in the
-    manner in which the model variable units have been determined. Note, parent is the compartment object.
-    example: Compartment('eCell', dimension=3, size=extraSize, parent=None)
-    """
+    """Model component representing a bounded reaction volume."""
 
     def __init__(self, name, parent=None, dimension=3, size=None, _export=True):
+        """
+        Requires name, accepts optional parent, dimension and size. name is a
+        string. parent should be the parent compartment, except for the root
+        compartment which should omit the parent argument. dimension may be 2
+        (for membranes) or 3 (for volumes). size is a parameter which defines
+        the compartment volume (the appropriate units will depend on the units
+        of the reaction rate constants).
+
+        Examples:
+        Compartment('cytosol', dimension=3, size=cyto_vol, parent=ec_membrane)
+        """
+
         Component.__init__(self, name, _export)
 
         if parent != None and isinstance(parent, Compartment) == False:
@@ -609,35 +676,19 @@ class Compartment(Component):
 
 class Rule(Component):
 
-    def __init__(self, name, reaction_pattern_set, rate_forward, rate_reverse=None,
+    def __init__(self, name, rule_expression, rate_forward, rate_reverse=None,
                  delete_molecules=False,
                  _export=True):
         Component.__init__(self, name, _export)
-
-        # FIXME: This tuple thing is ugly (used to support >> and <> operators between ReactionPatterns).
-        # This is how the reactant and product ReactionPatterns are passed, along with is_reversible.
-        if not isinstance(reaction_pattern_set, tuple) and len(reaction_pattern_set) != 3:
-            raise Exception("reaction_pattern_set must be a tuple of (ReactionPattern, ReactionPattern, Boolean)")
-
-        try:
-            reactant_pattern = as_reaction_pattern(reaction_pattern_set[0])
-        except InvalidReactionPatternException as e:
-            raise type(e)("Reactant does not look like a reaction pattern")
-
-        try:
-            product_pattern = as_reaction_pattern(reaction_pattern_set[1])
-        except InvalidReactionPatternException as e:
-            raise type(e)("Product does not look like a reaction pattern")
-
-        self.is_reversible = reaction_pattern_set[2]
-
+        if not isinstance(rule_expression, RuleExpression):
+            raise Exception("rule_expression is not a RuleExpression object")
         if not isinstance(rate_forward, Parameter):
             raise Exception("Forward rate must be a Parameter")
-        if self.is_reversible and not isinstance(rate_reverse, Parameter):
+        if rule_expression.is_reversible and not isinstance(rate_reverse, Parameter):
             raise Exception("Reverse rate must be a Parameter")
-
-        self.reactant_pattern = reactant_pattern
-        self.product_pattern = product_pattern
+        self.reactant_pattern = rule_expression.reactant_pattern
+        self.product_pattern = rule_expression.product_pattern
+        self.is_reversible = rule_expression.is_reversible
         self.rate_forward = rate_forward
         self.rate_reverse = rate_reverse
         self.delete_molecules = delete_molecules
